-- Imports required Turbine and custom CombatAlert modules
import "Turbine"
import "Turbine.Gameplay"
import "Turbine.UI"
import "Turbine.UI.Lotro"
import "CombatAlert.Utils"
import "CombatAlert.Settings"

-- Defines the module locally
local AlertUI = {}

-- Local reference to the main CombatAlert window
local CA

-- Reference to the alert text label
local alertLabel

-- Default dimensions for alert label
local baseWidth, baseHeight = 1200, 30

-- Sets the visibility of all alert components
function AlertUI.setAlertVisibility(visible, mouseVisible)
    if CA then
        CA:SetVisible(visible)
        CA:SetMouseVisible(mouseVisible)
        if CA.alertImgWin then
            CA.alertImgWin:SetVisible(visible)
            CA.alertImgWin:SetMouseVisible(mouseVisible)
        end
        if CA.alertDisplay then
            CA.alertDisplay:SetMouseVisible(mouseVisible)
        end
        if alertLabel then
            alertLabel:SetMouseVisible(mouseVisible)
        end
    end
end

-- Configures the alert display based on combat state and settings
function AlertUI.setupAlertDisplay(isEnterCombat, visualAlert)
    if CA and CA.alertDisplay then
        CA.alertDisplay:SetVisible(isEnterCombat and visualAlert)
    end
end

-- Updates the alert label size and position
local function updateLabelSize(zoomFactor)
    if alertLabel and CA then
        zoomFactor = zoomFactor or 100
        local selectedFont = _G.CombatAlert.Settings.get("alertFont") or "Default"
        local zoomModifier = _G.CombatAlert.Options.Fonts.getZoomModifier(selectedFont)
        local adjustedZoom = zoomFactor * zoomModifier

        -- Get the current text
        local currentText = alertLabel:GetText()

        -- Set the base size first
        alertLabel:SetSize(baseWidth, baseHeight)
        alertLabel:SetText(currentText)

        -- Calculate new dimensions based on zoom
        local labelWidth = baseWidth * (adjustedZoom / 100)
        local labelHeight = baseHeight * (adjustedZoom / 100)

        local savedX = tonumber(_G.CombatAlert.Settings.get("alertX"))
        local savedY = tonumber(_G.CombatAlert.Settings.get("alertY"))
        local newX, newY

        if savedX and savedY then
            newX, newY = savedX, savedY - (labelHeight / 2)
        else
            newX, newY = AlertUI.getCenteredAlertPosition()
        end

        -- Apply stretch mode and set the new size
        alertLabel:SetStretchMode(1)
        alertLabel:SetSize(labelWidth, labelHeight)
        alertLabel:SetText(currentText)
        alertLabel:SetPosition(newX - (labelWidth / 2), newY)

        _G.CombatAlert.Settings.set("alertX", newX)
        _G.CombatAlert.Settings.set("alertY", newY + (labelHeight / 2))
    end
end


-- Configures the alert label with text, color, and zoom settings
function AlertUI.setupAlertLabel(textAlert, color, text, zoomFactor)
    if alertLabel then
        alertLabel:SetVisible(textAlert)
        if textAlert then
            _G.CombatAlert.Utils.applyColor(alertLabel, color)
            alertLabel:SetText(text)
            updateLabelSize(zoomFactor)
        end
    end
end

-- Sets the reference to the alert label
function AlertUI.setLabel(label)
    alertLabel = label
end

-- Updates the text zoom and font of the alert label
function AlertUI.updateTextZoom(zoom)
    if alertLabel then
        local selectedFont = _G.CombatAlert.Settings.get("alertFont") or
            "Default"
        local fontObject = _G.CombatAlert.Options.Fonts.fontMapping
            [selectedFont]

        -- Set the font based on the selected font
        local font
        if fontObject and fontObject.font then
            font = fontObject.font
        else
            -- Default to TrajanProBold if the font is not found
            font = Turbine.UI.Lotro.Font.TrajanProBold24
        end

        -- Apply the font
        alertLabel:SetFont(font)

        -- Set the font style and outline color
        alertLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
        alertLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))

        -- Force a redraw of the text
        alertLabel:SetText(alertLabel:GetText())

        updateLabelSize(zoom)
        _G.CombatAlert.Settings.set("textZoom", zoom)
    end
end

-- Updates the alert background image and opacity
function AlertUI.updateAlertBackground()
    if CA and CA.alertDisplay then
        local settings = _G.CombatAlert.Settings.get()
        local opacity = settings.opacity or 255
        local alertColor = settings.alertColor or "Default"
        local color

        if alertColor == "Custom" then
            color = settings.alertColorPresets.Custom
        else
            -- Retrieves color from presets, ensuring it exists
            color = settings.alertColorPresets[alertColor]
            if not color then
                color = {1, 1, 1, 1}
            end
        end

        -- Always sets alpha to 1 to replace original color in the image
        color[1] = 1

        -- Sets the background image
        CA.alertDisplay:SetBackground(
            "CombatAlert/Resources/AlertImage-Default.tga")

        if alertColor == "Default" then
            -- Disables blend mode for Default
            CA.alertDisplay:SetBackColor(Turbine.UI.Color(0, 0, 0, 0))
            CA.alertDisplay:SetBackColorBlendMode(Turbine.UI.BlendMode.None)
        else
            -- Applies colorization
            CA.alertDisplay:SetBackColor(Turbine.UI.Color(
                color[1], color[2], color[3], color[4]
            ))
            CA.alertDisplay:SetBackColorBlendMode(Turbine.UI.BlendMode.Color)
        end

        -- Sets opacity
        CA.alertDisplay:SetOpacity(opacity / 255)
    end
end

-- Updates the layout of the alert UI components
function AlertUI.updateLayout()
    if CA then
        CA.winWidth, CA.winHeight = Turbine.UI.Display:GetSize()
        CA:SetSize(CA.winWidth, CA.winHeight)
        local currentZoom = tonumber(
            _G.CombatAlert.Settings.get("textZoom")
        ) or 100
        updateLabelSize(currentZoom)
    end
end

-- Updates the position of the alert label
function AlertUI.updateAlertPosition(x, y)
    if alertLabel then
        local labelWidth, labelHeight = alertLabel:GetSize()
        alertLabel:SetPosition(x - (labelWidth / 2), y - (labelHeight / 2))
        _G.CombatAlert.Settings.set("alertX", x)
        _G.CombatAlert.Settings.set("alertY", y)
        _G.CombatAlert.AlertLogic.previewAlert(false, true)
    end
end

-- Gets the size of the alert window
function AlertUI.getAlertWindowSize()
    if CA and CA.alertImgWin then
        return CA.alertImgWin:GetSize()
    end
    return Turbine.UI.Display:GetSize()
end

-- Gets the size of the alert label
function AlertUI.getAlertLabelSize()
    if alertLabel then
        return alertLabel:GetSize()
    end
    return baseWidth, baseHeight
end

-- Gets the default X position for the alert
function AlertUI.getDefaultX()
    local windowWidth, _ = AlertUI.getAlertWindowSize()
    local labelWidth, _ = AlertUI.getAlertLabelSize()
    return (windowWidth - labelWidth) / 2
end

-- Gets the default Y position for the alert
function AlertUI.getDefaultY()
    local _, windowHeight = AlertUI.getAlertWindowSize()
    local _, labelHeight = AlertUI.getAlertLabelSize()
    return (windowHeight / 4) - (labelHeight / 2)
end

-- Gets the centered position for the alert
function AlertUI.getCenteredAlertPosition()
    local windowWidth, windowHeight = AlertUI.getAlertWindowSize()
    local labelWidth, labelHeight = AlertUI.getAlertLabelSize()
    local centeredX = windowWidth / 2
    local centeredY = (windowHeight / 4) - (labelHeight / 2)
    return centeredX, centeredY
end

-- Initializes the AlertUI module
function AlertUI.initialize(caInstance)
    CA = caInstance
    
    -- Set the window size to the full screen size
    local screenWidth, screenHeight = Turbine.UI.Display:GetSize()
    CA:SetSize(screenWidth, screenHeight)

    local savedZoom = _G.CombatAlert.Settings.get("textZoom")
    if savedZoom then
        AlertUI.updateTextZoom(savedZoom)
    else
        AlertUI.updateTextZoom(100)
    end

    local savedX = tonumber(_G.CombatAlert.Settings.get("alertX"))
    local savedY = tonumber(_G.CombatAlert.Settings.get("alertY"))
    if savedX and savedY then
        AlertUI.updateAlertPosition(savedX, savedY)
    else
        local defaultX, defaultY = AlertUI.getDefaultX(), AlertUI.getDefaultY()
        AlertUI.updateAlertPosition(defaultX, defaultY)
    end
end

-- Makes AlertUI globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.AlertUI = AlertUI

return AlertUI
