-- Imports required Turbine and custom CombatAlert modules
import "Turbine"
import "Turbine.Gameplay"
import "Turbine.UI"
import "Turbine.UI.Lotro"
import "CombatAlert.Utils"
import "CombatAlert.Settings"
import "CombatAlert.AlertLogic"
import "CombatAlert.AlertUI"
import "CombatAlert.Options"

-- Defines the module locally
local Main = {}

-- Initializes global namespace and flag
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.isInitializing = true

-- Initializes and configures the main plugin window
Main.window = Turbine.UI.Window()
local CA = Main.window
CA.winWidth, CA.winHeight = Turbine.UI.Display:GetSize()
CA:SetPosition(0, 0)
CA:SetSize(CA.winWidth, CA.winHeight)
CA:SetVisible(false)
CA:SetBackColor(Turbine.UI.Color(0, 0, 0, 0))

-- Sets up window resize handler
CA.sizeChanged = function(sender, args)
    CA.winWidth, CA.winHeight = sender:GetSize()
    _G.CombatAlert.AlertUI.updateLayout()
end

-- Creates and configures alert image window
CA.alertImgWin = Turbine.UI.Window()
CA.alertImgWin:SetParent(CA)
CA.alertImgWin:SetPosition(0, 0)
CA.alertImgWin:SetSize(CA.winWidth, CA.winHeight)
CA.alertImgWin:SetBackColor(Turbine.UI.Color(0, 0, 0, 0))
CA.alertImgWin:SetVisible(false)

-- Loads user settings
_G.CombatAlert.settings = _G.CombatAlert.Settings.load()

-- Loads background image for visual alerts
local function loadBackgroundImage()
    local bgPaths = {
        _G.CombatAlert.settings.alertImageFile
    }

    for _, path in ipairs(bgPaths) do
        local success, image = _G.CombatAlert.Utils.tryLoadBackground(
            CA.alertImgWin, path)
        if success then return image end
    end
    return nil
end

CA.alertDisplay = loadBackgroundImage()

-- Sets up text label for alerts
CA.label = Turbine.UI.Label()
CA.label:SetParent(CA)
CA.label:SetSize(300, 30)
CA.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
CA.label:SetFont(Turbine.UI.Lotro.Font.TrajanProBold36)
CA.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
CA.label:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
CA.label:SetOpacity(1)

-- Initializes player instance and sets up AlertUI
_G.CombatAlert.player = Turbine.Gameplay.LocalPlayer.GetInstance()
_G.CombatAlert.AlertUI.setLabel(CA.label)

-- Initializes AlertLogic and AlertUI modules
_G.CombatAlert.AlertLogic.initialize(CA)
_G.CombatAlert.AlertUI.initialize(CA)

-- Applies initial position and zoom
local savedX, savedY = _G.CombatAlert.settings.alertX,
                       _G.CombatAlert.settings.alertY
if savedX and savedY then
    _G.CombatAlert.AlertUI.updateAlertPosition(savedX, savedY)
else
    local centeredX, centeredY =
        _G.CombatAlert.AlertUI.getCenteredAlertPosition()
    _G.CombatAlert.AlertUI.updateAlertPosition(centeredX, centeredY)
end

local initialZoom = math.min(math.max(
    tonumber(_G.CombatAlert.settings.textZoom) or 100, 50), 1800)
_G.CombatAlert.AlertUI.updateTextZoom(initialZoom)

-- Updates initial alert background
_G.CombatAlert.AlertUI.updateAlertBackground()

-- Registers combat change event handler
_G.CombatAlert.Utils.addCallback(_G.CombatAlert.player, "InCombatChanged",
    _G.CombatAlert.AlertLogic.combatChanged)

-- Creates and sets up options panel
CA.optPanel = _G.CombatAlert.Options.OptionsPanel()
plugin.GetOptionsPanel = function() return CA.optPanel end

-- Cleans up function to remove event handlers
Main.cleanup = function()
    _G.CombatAlert.Utils.removeCallback(_G.CombatAlert.player,
        "InCombatChanged", _G.CombatAlert.AlertLogic.combatChanged)
end

-- Registers cleanup function for plugin unload
plugin.Unload = Main.cleanup

-- Updates the layout after initialization
_G.CombatAlert.AlertUI.updateLayout()

-- Forces update of text zoom
local function forceUpdateTextZoom()
    local currentZoom = tonumber(_G.CombatAlert.Settings.get("textZoom"))
                        or 100
    _G.CombatAlert.AlertUI.updateTextZoom(currentZoom)
end

-- Controls to handle delayed update
local updateControl = Turbine.UI.Control()
updateControl:SetWantsUpdates(true)
updateControl.Update = function()
    forceUpdateTextZoom()
    updateControl:SetWantsUpdates(false)
end

-- Resets to default centered position
Main.resetToDefaultPosition = function()
    local centeredX, centeredY =
        _G.CombatAlert.AlertUI.getCenteredAlertPosition()
    _G.CombatAlert.AlertUI.updateAlertPosition(centeredX, centeredY)
    _G.CombatAlert.Settings.set("alertX", centeredX)
    _G.CombatAlert.Settings.set("alertY", centeredY)
end

-- Makes Main globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Main = Main

-- Clears the initialization flag
_G.CombatAlert.isInitializing = false

return Main
