-- Defines the module locally
local Controls = {}

-- Creates a label
function Controls.createLabel(parent, size, position, text, alignment)
    local label = Turbine.UI.Label()
    label:SetParent(parent)
    label:SetSize(size[1], size[2])
    label:SetFont(Turbine.UI.Lotro.Font.TrajanPro14)
    label:SetForeColor(Turbine.UI.Color(0.8, 0.75, 0.5))
    label:SetPosition(position[1], position[2])
    label:SetText(text)
    label:SetTextAlignment(alignment or
                           Turbine.UI.ContentAlignment.MiddleLeft)
    return label
end

-- Creates a slider
function Controls.createSlider(parent, label, index, initialValue, hOffset)
    local yOffset = (index - 1) * 22 + 35
    local slider = Turbine.UI.Lotro.ScrollBar()
    slider:SetParent(parent)
    local sliderWidth = 250
    slider:SetSize(sliderWidth, 10)
    slider:SetOrientation(Turbine.UI.Orientation.Horizontal)
    slider:SetPosition(hOffset, yOffset)
    slider:SetMinimum(0)
    slider:SetMaximum(255)

    initialValue = tonumber(initialValue) or 0
    slider:SetValue(math.max(0, math.min(255, initialValue)))

    local sliderLabel = Controls.createLabel(
        parent, {150, 20}, {hOffset + sliderWidth + 10, yOffset - 5},
        label .. ": " .. tostring(initialValue)
    )

    local function updateLabel()
        sliderLabel:SetText(label .. ": " .. tostring(slider:GetValue()))
    end

    slider.ValueChanged = function()
        updateLabel()
    end

    return slider, updateLabel
end

-- Creates a button
function Controls.createButton(parent, size, position, text, clickFunction)
    local button = Turbine.UI.Lotro.Button()
    button:SetParent(parent)
    button:SetSize(size[1], size[2])
    button:SetPosition(position[1], position[2])
    button:SetText(text)
    button.Click = clickFunction
    return button
end

-- Creates a checkbox
function Controls.createCheckBox(parent, text, position, settingKey)
    local checkBox = Turbine.UI.Lotro.CheckBox()
    checkBox:SetParent(parent)
    checkBox:SetFont(Turbine.UI.Lotro.Font.TrajanPro14)
    checkBox:SetForeColor(Turbine.UI.Color(0.8, 0.75, 0.5))
    checkBox:SetSize(336, 20)
    checkBox:SetPosition(position.x, position.y)
    checkBox:SetText(text .. " ")
    checkBox:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    checkBox:SetChecked(_G.CombatAlert.Settings.get(settingKey))

    checkBox.CheckedChanged = function(sender, args)
        _G.CombatAlert.Settings.set(settingKey, checkBox:IsChecked())
    end

    return checkBox
end

-- Creates a text zoom slider
function Controls.createTextZoomSlider(parent, position)
    local textLabel = Controls.createLabel(
        parent, {80, 20}, {10, position}, "Text Zoom:"
    )
    local slider = Turbine.UI.Lotro.ScrollBar()
    slider:SetParent(parent)
    slider:SetSize(218, 10)
    slider:SetOrientation(Turbine.UI.Orientation.Horizontal)
    slider:SetPosition(95, position + 5)
    slider:SetMinimum(50)
    slider:SetMaximum(1800)
    slider:SetValue(_G.CombatAlert.Settings.get("textZoom") or 100)
    local valueLabel = Controls.createLabel(
        parent, {50, 20}, {320, position}, slider:GetValue() .. "%"
    )

    local function updateZoom(zoomValue, triggerPreview)
        zoomValue = tonumber(zoomValue) or 100
        slider:SetValue(zoomValue)
        valueLabel:SetText(tostring(zoomValue) .. "%")
        _G.CombatAlert.Settings.set("textZoom", zoomValue)
        _G.CombatAlert.AlertUI.updateTextZoom(zoomValue)

        if triggerPreview then
            _G.CombatAlert.AlertLogic.previewAlert(false, true)
        end
    end

    local isManualAdjustment = true
    slider.ValueChanged = function(sender, args)
        if isManualAdjustment then
            updateZoom(slider:GetValue(), true)
        end
    end

    local function setZoomWithoutPreview(value)
        isManualAdjustment = false
        updateZoom(value, false)
        isManualAdjustment = true
    end

    setZoomWithoutPreview(
        tonumber(_G.CombatAlert.Settings.get("textZoom")) or 100
    )

    return slider, valueLabel, setZoomWithoutPreview
end

-- Creates a dropdown
function Controls.createDropdown(parent, label, position, width)
    local dropdownWidth = width or 200
    local colorLabel = Controls.createLabel(
        parent, {136, 20}, {position.x, position.y}, label
    )

    local dropdown = Turbine.UI.Control()
    dropdown:SetParent(parent)
    dropdown:SetSize(dropdownWidth, 20)
    dropdown:SetPosition(position.x + 146, position.y)
    dropdown:SetBackColor(Turbine.UI.Color(0.2, 0.2, 0.2))

    local selectedValueLabel = Controls.createLabel(
        dropdown, {dropdownWidth - 20, 20}, {5, 0}, "Default",
        Turbine.UI.ContentAlignment.MiddleLeft
    )

    local dropDownArrow = Turbine.UI.Control()
    dropDownArrow:SetParent(dropdown)
    dropDownArrow:SetSize(16, 16)
    dropDownArrow:SetPosition(dropdownWidth - 16, 2)
    dropDownArrow:SetBackground("CombatAlert/Resources/DropdownArrow.tga")

    -- Populates dropdown with color options
    local colorOptions = {
        "Default", "Custom", "Blue", "Cyan", "Pink", "Purple",
        "Red", "Green", "Silver", "Gold", "Orange"
    }

    return dropdown, dropDownArrow, colorLabel, selectedValueLabel
end

-- Creates a preview button
function Controls.createPreviewButton(parent, text, position, clickFunction)
    return Controls.createButton(
        parent, {104, 20}, {position.x, position.y}, text, clickFunction
    )
end

-- Creates a load defaults button
function Controls.createLoadDefaultsButton(parent, position)
    return Controls.createButton(
        parent, {104, 20}, {position.x, position.y}, "Load Defaults",
        function(sender, args)
            local defaults = _G.CombatAlert.Settings.resetToDefaults()
            parent.enterCB:SetChecked(defaults.entercombatalert)
            parent.leaveCB:SetChecked(defaults.leavecombatalert)
            parent.visualCB:SetChecked(defaults.visualalert)

            local originalValueChanged = parent.textZoomSlider.ValueChanged
            parent.textZoomSlider.ValueChanged = nil
            parent.textZoomSlider:SetValue(defaults.textZoom)
            parent.textZoomLabel:SetText(tostring(defaults.textZoom) .. "%")
            parent.textZoomSlider.ValueChanged = originalValueChanged

            if parent.colorOptions and parent.colorOptions.updateColors then
                parent.colorOptions:updateColors(
                    defaults.enterColor, defaults.leaveColor
                )
            end

            if parent.selectedValueLabel then
                parent.selectedValueLabel:SetText("Default")
            end

            _G.CombatAlert.AlertUI.updateTextZoom(defaults.textZoom)
            _G.CombatAlert.AlertUI.updateAlertBackground()
        end
    )
end

-- Makes Controls globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Options = _G.CombatAlert.Options or {}
_G.CombatAlert.Options.Controls = Controls

return Controls
