import "Turbine.UI.Lotro"

local OptionsPanel = class(Turbine.UI.Control)

-- Accesses the TabManager class from the global namespace
local TabManager = _G.CombatAlert.Options.Tabs

-- Function to update label size based on text content
function OptionsPanel:updateLabelSize(label, text)
    if not label then
        -- Log an error or handle the case where label is nil
        Turbine.Shell.WriteLine("Error: Attempted to update size of nil label")
        return
    end

    local font = label:GetFont()
    if not font then
        -- Log an error or handle the case where font is nil
        Turbine.Shell.WriteLine("Error: Label has no font")
        return
    end

    local width = Turbine.UI.Display.GetTextWidth(font, text)
    label:SetSize(width, label:GetHeight())
end

-- Constructs a new OptionsPanel
function OptionsPanel:Constructor()
    Turbine.UI.Control.Constructor(self)

    -- Sets the background color of the options panel
    self:SetBackColor(Turbine.UI.Color(0.05, 0.05, 0.1))

    -- Initializes the tab manager to manage different settings tabs
    self.tabManager = TabManager()
    self.tabManager:SetParent(self)
    self.tabManager:SetPosition(0, 0)
    self.tabManager:SetSize(520, 546)  -- Increased height by 60 pixels

    -- Creates a container for the buttons at the bottom of the panel
    self.buttonContainer = Turbine.UI.Control()
    self.buttonContainer:SetParent(self)
    self.buttonContainer:SetPosition(1, 546)  -- Moved down by 60 pixels
    self.buttonContainer:SetSize(518, 30)
    self.buttonContainer:SetBackColor(Turbine.UI.Color(0.05, 0.05, 0.1))

    -- Creates content panels for text and visual alerts
    local textAlertsPanel = Turbine.UI.Control()
    textAlertsPanel:SetBackColor(Turbine.UI.Color(0.05, 0.05, 0.1))

    local visualAlertsPanel = Turbine.UI.Control()
    visualAlertsPanel:SetBackColor(Turbine.UI.Color(0.05, 0.05, 0.1))

    -- Adds tabs to the tab manager for text and visual alerts
    self.tabManager:addTab("Text", textAlertsPanel)
    self.tabManager:addTab("Visual", visualAlertsPanel)

    -- Sets the size and position of the options panel
    self:SetSize(520, 660)  -- Increased height by 60 pixels
    self:SetPosition(0, 0)

    -- Configures layout adjustments when the panel size changes
    self.SizeChanged = function(sender, args)
        self:configureLayout()
    end

    -- Defines button dimensions and positions
    local buttonWidth = 104
    local buttonGap = 14
    local buttonYPosition = 5

    -- Creates a button to preview the enter combat alert
    self.previewEnterButton = _G.CombatAlert.Options.Controls
        .createPreviewButton(
            self.buttonContainer,
            "Preview Enter",
            {x = 10, y = buttonYPosition},
            function(sender, args)
                _G.CombatAlert.AlertLogic.previewAlert(true)
            end
        )

    -- Creates a button to preview the leave combat alert
    self.previewLeaveButton = _G.CombatAlert.Options.Controls
        .createPreviewButton(
            self.buttonContainer,
            "Preview Leave",
            {x = 10 + buttonWidth + buttonGap, y = buttonYPosition},
            function(sender, args)
                _G.CombatAlert.AlertLogic.previewAlert(false)
            end
        )

    -- Creates a button to load default settings
    self.loadDefaultsButton = Turbine.UI.Lotro.Button()
    self.loadDefaultsButton:SetParent(self.buttonContainer)
    self.loadDefaultsButton:SetSize(104, 20)
    self.loadDefaultsButton:SetPosition(
        10 + (buttonWidth + buttonGap) * 2,
        buttonYPosition
    )
    -- Sets button text and click event for loading defaults
    self.loadDefaultsButton:SetText("Load Defaults")
    self.loadDefaultsButton.Click = function(sender, args)
        -- Resets settings to defaults and updates UI components
        local defaults = _G.CombatAlert.Settings.resetToDefaults()
        self.enterCB:SetChecked(defaults.entercombatalert)
        self.leaveCB:SetChecked(defaults.leavecombatalert)
        self.visualCB:SetChecked(defaults.visualalert)

        self:updateTextZoom(defaults.textZoom, false)

        if self.colorOptions and self.colorOptions.updateColors then
            self.colorOptions:updateColors(
                defaults.enterColor,
                defaults.leaveColor
            )
        end

        if self.selectedValueLabel then
            self.selectedValueLabel:SetText("Default")
        end

        -- Update visual alert sliders without changing to Custom
        if self.customColorOptions then
            local defaultColor = defaults.alertColorPresets.Default
            _G.CombatAlert.Options.VisualAlerts.loadDefaults(defaultColor)
        end

        _G.CombatAlert.Settings.set("alertColor", "Default")
        _G.CombatAlert.AlertUI.updateAlertBackground()

        local centeredX, centeredY = _G.CombatAlert.AlertUI
            .getCenteredAlertPosition()
        self:updateAlertPosition(centeredX, centeredY)

        -- Set default font
        if self.fontSelectedValueLabel then
            self.fontSelectedValueLabel:SetText("Default")
        end
        _G.CombatAlert.Options.Fonts.setFont("Default")

        -- Reset Enter and Leave Combat Text
        if self.enterTextInput then
            self.enterTextInput:SetText(defaults.enterCombatText)
            if _G.CombatAlert.AlertUI and _G.CombatAlert.AlertUI.enterLabel then
                self:updateLabelSize(
                    _G.CombatAlert.AlertUI.enterLabel,
                    defaults.enterCombatText
                )
            end
        end
        if self.leaveTextInput then
            self.leaveTextInput:SetText(defaults.leaveCombatText)
            if _G.CombatAlert.AlertUI and _G.CombatAlert.AlertUI.leaveLabel then
                self:updateLabelSize(
                    _G.CombatAlert.AlertUI.leaveLabel,
                    defaults.leaveCombatText
                )
            end
        end

        -- Trigger enter combat preview
        _G.CombatAlert.AlertLogic.previewAlert(true)
    end

    -- Initializes the options panel with various settings and controls
    self:createOptionsPanel()
end

-- Configures the layout of the OptionsPanel
function OptionsPanel:configureLayout()
    local width, height = self:GetSize()
    -- Adjusts tab manager size and button container position
    self.tabManager:SetSize(width - 2, height - 114)
    self.buttonContainer:SetPosition(1, height - 114)
    self.buttonContainer:SetWidth(width - 2)
end

-- Updates the text zoom level in the UI and settings
function OptionsPanel:updateTextZoom(zoom, triggerPreview)
    if self.textZoomSlider then
        self.textZoomSlider:SetValue(zoom)
    end
    if self.textZoomLabel then
        self.textZoomLabel:SetText(tostring(zoom) .. "%")
    end
    -- Updates zoom in AlertUI and Settings
    _G.CombatAlert.AlertUI.updateTextZoom(zoom)
    _G.CombatAlert.Settings.set("textZoom", zoom)

    if triggerPreview then
        _G.CombatAlert.AlertLogic.previewAlert(false)
    end
end

-- Updates the alert position in the UI and sliders
function OptionsPanel:updateAlertPosition(x, y)
    _G.CombatAlert.AlertUI.updateAlertPosition(x, y)
    self.xSlider:SetValue(x)
    self.ySlider:SetValue(y)
end

-- Gets the centered alert position for resetting
function OptionsPanel:getCenteredAlertPosition()
    return _G.CombatAlert.AlertUI.getCenteredAlertPosition()
end

-- Creates the options panel with various settings and controls
function OptionsPanel:createOptionsPanel()
    local textAlertsContent = self.tabManager.tabs[1].content
    local visualAlertsContent = self.tabManager.tabs[2].content

    -- Text Alerts Tab
    -- Sets up font dropdown at the top of the text tab
    local fontDropdownPosition = {x = 10, y = 10}
    self.fontDropdown, self.fontDropDownArrow, self.fontLabel,
    self.fontSelectedValueLabel, self.fontListBox =
        _G.CombatAlert.Options.Fonts.createFontDropdown(
            textAlertsContent, fontDropdownPosition
        )

    -- Get the width of the dropdown control
    local dropdownWidth = self.fontDropdown:GetWidth()

    -- Sets up checkboxes for text alerts when entering and leaving combat
    local basePosition = 10
    local verticalSpacing = 30

    self.enterCB = _G.CombatAlert.Options.Controls.createCheckBox(
        textAlertsContent,
        "Display text alert when entering combat",
        {x = basePosition, y = basePosition + 30},
        "entercombatalert"
    )

    self.leaveCB = _G.CombatAlert.Options.Controls.createCheckBox(
        textAlertsContent,
        "Display text alert when leaving combat",
        {x = basePosition, y = basePosition + 30 + verticalSpacing},
        "leavecombatalert"
    )
    -- Add Enter Combat Text input
    local enterTextLabel = _G.CombatAlert.Options.Controls.createLabel(
        textAlertsContent,
        {140, 20},
        {basePosition, basePosition + 30 + (verticalSpacing * 2)},
        "Enter Combat Text:"
    )
    enterTextLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)

    self.enterTextInput = Turbine.UI.Lotro.TextBox()
    self.enterTextInput:SetParent(textAlertsContent)
    self.enterTextInput:SetSize(dropdownWidth, 20)
    self.enterTextInput:SetPosition(
        self.fontDropdown:GetLeft(),
        basePosition + 30 + (verticalSpacing * 2)
    )
    self.enterTextInput:SetText(
        _G.CombatAlert.Settings.get("enterCombatText") or "Entering Combat"
    )
    self.enterTextInput:SetFont(Turbine.UI.Lotro.Font.Verdana14)
    self.enterTextInput:SetForeColor(Turbine.UI.Color(0.75, 0.75, 0.75))
    self.enterTextInput:SetMultiline(false)

    -- Add Leave Combat Text input
    local leaveTextLabel = _G.CombatAlert.Options.Controls.createLabel(
        textAlertsContent,
        {140, 20},
        {basePosition, basePosition + 30 + (verticalSpacing * 3)},
        "Leave Combat Text:"
    )
    leaveTextLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)

    self.leaveTextInput = Turbine.UI.Lotro.TextBox()
    self.leaveTextInput:SetParent(textAlertsContent)
    self.leaveTextInput:SetSize(dropdownWidth, 20)
    self.leaveTextInput:SetPosition(
        self.fontDropdown:GetLeft(),
        basePosition + 30 + (verticalSpacing * 3)
    )
    self.leaveTextInput:SetText(
        _G.CombatAlert.Settings.get("leaveCombatText") or "Leaving Combat"
    )
    self.leaveTextInput:SetFont(Turbine.UI.Lotro.Font.Verdana14)
    self.leaveTextInput:SetForeColor(Turbine.UI.Color(0.75, 0.75, 0.75))
    self.leaveTextInput:SetMultiline(false)

    -- Event handlers for text inputs
    self.enterTextInput.TextChanged = function(sender)
        local newText = sender:GetText()
        _G.CombatAlert.Settings.set("enterCombatText", newText)
        if _G.CombatAlert.AlertUI and _G.CombatAlert.AlertUI.enterLabel then
            self:updateLabelSize(_G.CombatAlert.AlertUI.enterLabel, newText)
        end
        _G.CombatAlert.AlertLogic.previewAlert(true)
    end

    self.leaveTextInput.TextChanged = function(sender)
        local newText = sender:GetText()
        _G.CombatAlert.Settings.set("leaveCombatText", newText)
        if _G.CombatAlert.AlertUI and _G.CombatAlert.AlertUI.leaveLabel then
            self:updateLabelSize(_G.CombatAlert.AlertUI.leaveLabel, newText)
        end
        _G.CombatAlert.AlertLogic.previewAlert(false)
    end

    -- Prevent Enter key from adding new lines
    self.enterTextInput.KeyDown = function(sender, args)
        if args.Action == Turbine.UI.Lotro.Action.EnterKey then
            return true
        end
    end

    self.leaveTextInput.KeyDown = function(sender, args)
        if args.Action == Turbine.UI.Lotro.Action.EnterKey then
            return true
        end
    end

    -- Initial update of label sizes
    if _G.CombatAlert.AlertUI then
        if _G.CombatAlert.AlertUI.enterLabel then
            self:updateLabelSize(
                _G.CombatAlert.AlertUI.enterLabel,
                self.enterTextInput:GetText()
            )
        end
        if _G.CombatAlert.AlertUI.leaveLabel then
            self:updateLabelSize(
                _G.CombatAlert.AlertUI.leaveLabel,
                self.leaveTextInput:GetText()
            )
        end
    end

    -- Creates a slider for adjusting text zoom
    local textZoomPosition = basePosition + 30 + (verticalSpacing * 4)
    self.textZoomSlider, self.textZoomLabel, self.setZoomWithoutPreview =
        _G.CombatAlert.Options.TextAlerts.createTextZoomSlider(
            textAlertsContent,
            textZoomPosition
        )

    -- Adds X and Y sliders and a Reset button for alert position
    local xySliderPosition = textZoomPosition + verticalSpacing
    local sliderWidth = 131
    local resetButtonWidth = 56
    local gap = 10
    local labelWidth = 20

    -- Creates X slider and label
    self.xLabel = _G.CombatAlert.Options.Controls.createLabel(
        textAlertsContent,
        {labelWidth, 20},
        {basePosition, xySliderPosition},
        "X"
    )
    self.xLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)

    self.xSlider = Turbine.UI.Lotro.ScrollBar()
    self.xSlider:SetParent(textAlertsContent)
    self.xSlider:SetOrientation(Turbine.UI.Orientation.Horizontal)
    self.xSlider:SetSize(sliderWidth - labelWidth, 10)
    self.xSlider:SetPosition(
        basePosition + labelWidth,
        xySliderPosition + 5
    )
    self.xSlider:SetMinimum(0)
    self.xSlider:SetMaximum(Turbine.UI.Display:GetWidth())
    self.xSlider:SetValue(
        _G.CombatAlert.Settings.get("alertX") or
        _G.CombatAlert.AlertUI.getDefaultX()
    )

    -- Creates Reset button to reset alert position
    self.resetButton = Turbine.UI.Lotro.Button()
    self.resetButton:SetParent(textAlertsContent)
    self.resetButton:SetSize(resetButtonWidth, 20)
    self.resetButton:SetPosition(
        basePosition + sliderWidth + gap,
        xySliderPosition
    )
    self.resetButton:SetText("Reset")

    -- Creates Y slider and label
    self.ySlider = Turbine.UI.Lotro.ScrollBar()
    self.ySlider:SetParent(textAlertsContent)
    self.ySlider:SetOrientation(Turbine.UI.Orientation.Horizontal)
    self.ySlider:SetSize(sliderWidth - labelWidth, 10)
    local ySliderX = basePosition + sliderWidth + resetButtonWidth + (gap * 2)
    self.ySlider:SetPosition(ySliderX, xySliderPosition + 5)
    self.ySlider:SetMinimum(0)
    self.ySlider:SetMaximum(Turbine.UI.Display:GetHeight())
    self.ySlider:SetValue(
        _G.CombatAlert.Settings.get("alertY") or
        _G.CombatAlert.AlertUI.getDefaultY()
    )

    local yLabelX = ySliderX + sliderWidth - labelWidth
    self.yLabel = _G.CombatAlert.Options.Controls.createLabel(
        textAlertsContent,
        {labelWidth, 20},
        {yLabelX, xySliderPosition},
        "Y"
    )
    self.yLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)

    -- Sets up event handlers for sliders and reset button
    self.xSlider.ValueChanged = function(sender, args)
        _G.CombatAlert.AlertUI.updateAlertPosition(
            sender:GetValue(),
            self.ySlider:GetValue()
        )
    end

    self.ySlider.ValueChanged = function(sender, args)
        _G.CombatAlert.AlertUI.updateAlertPosition(
            self.xSlider:GetValue(),
            sender:GetValue()
        )
    end

    self.resetButton.Click = function(sender, args)
        local defaultX, defaultY = self:getCenteredAlertPosition()
        self:updateAlertPosition(defaultX, defaultY)
    end

    -- Creates color options for enter and leave combat alerts
    local colorOptionsPosition = xySliderPosition + verticalSpacing + 16
    local defaultSettings = _G.CombatAlert.Settings.defaults
    local defaultEnterColor = defaultSettings and defaultSettings.enterColor
    local defaultLeaveColor = defaultSettings and defaultSettings.leaveColor

    self.colorOptions = _G.CombatAlert.Options.TextColors.dualcolorOptions(
        textAlertsContent,
        480,
        "Enter Combat Color",
        "Leave Combat Color",
        _G.CombatAlert.Settings.get("enterColor") or defaultEnterColor,
        _G.CombatAlert.Settings.get("leaveColor") or defaultLeaveColor,
        function(newColor)
            _G.CombatAlert.Settings.set("enterColor", newColor)
        end,
        function(newColor)
            _G.CombatAlert.Settings.set("leaveColor", newColor)
        end
    )

    if self.colorOptions then
        self.colorOptions:SetParent(textAlertsContent)
        self.colorOptions:SetPosition(basePosition, colorOptionsPosition)
    end

    -- Sets up visual alert tab options
    local visualAlertOptions = _G.CombatAlert.Options.VisualAlerts
        .createVisualAlertsTab(visualAlertsContent)

    self.colorDropDown = visualAlertOptions.colorDropDown
    self.dropDownArrow = visualAlertOptions.dropDownArrow
    self.selectedValueLabel = visualAlertOptions.selectedValueLabel
    self.colorListBox = visualAlertOptions.colorListBox
    self.visualCB = visualAlertOptions.visualCB
    self.customColorOptions = visualAlertOptions.customColorOptions

    -- Sets up event handlers for dropdown interactions
    self.colorDropDown.MouseClick = function(sender, args)
        _G.CombatAlert.Options.VisualAlerts.toggleDropdown(self)
    end

    self.dropDownArrow.MouseClick = self.colorDropDown.MouseClick

    -- Function to update sliders based on current settings
    local function updateVisualAlertSliders()
        local alertColor = _G.CombatAlert.Settings.get("alertColor") or
                           "Default"
        local colorValues
        if alertColor == "Custom" then
            colorValues = _G.CombatAlert.Settings.get("alertColorPresets")
                .Custom
        else
            colorValues = _G.CombatAlert.Options.VisualAlerts
                .getPresetColor(alertColor)
        end
        _G.CombatAlert.Options.VisualAlerts.applyPresetColor(colorValues)
        self.selectedValueLabel:SetText(alertColor)
    end

    -- Store the updateVisualAlertSliders function for access from outside
    self.updateVisualAlertSliders = updateVisualAlertSliders

    -- Call updateVisualAlertSliders initially to set correct values
    updateVisualAlertSliders()

    -- Sets the initial zoom without triggering the preview
    local initialZoom = _G.CombatAlert.Settings.get("textZoom") or 100
    self:updateTextZoom(initialZoom, false) -- Ensure no preview is triggered

    -- Set up tab selection callback
    self.tabManager.OnTabSelected = function(tabIndex)
        if tabIndex == 2 then  -- Assuming Visual Alerts is the second tab
            updateVisualAlertSliders()
        end
    end

    -- Set initial font
    local initialFont = _G.CombatAlert.Settings.get("alertFont") or "Default"
    self.fontSelectedValueLabel:SetText(initialFont)
    _G.CombatAlert.Options.Fonts.setFont(initialFont)

    -- Set initial Enter and Leave Combat Text
    local initialEnterText = _G.CombatAlert.Settings.get("enterCombatText") or
        "Entering Combat"
    local initialLeaveText = _G.CombatAlert.Settings.get("leaveCombatText") or
        "Leaving Combat"
    self.enterTextInput:SetText(initialEnterText)
    self.leaveTextInput:SetText(initialLeaveText)

    -- Initial update of label sizes
    if _G.CombatAlert.AlertUI then
        if _G.CombatAlert.AlertUI.enterLabel then
            self:updateLabelSize(
                _G.CombatAlert.AlertUI.enterLabel,
                initialEnterText
            )
        end
        if _G.CombatAlert.AlertUI.leaveLabel then
            self:updateLabelSize(
                _G.CombatAlert.AlertUI.leaveLabel,
                initialLeaveText
            )
        end
    end
end

-- Make OptionsPanel accessible in the global CombatAlert namespace
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Options = _G.CombatAlert.Options or {}
_G.CombatAlert.Options.OptionsPanel = OptionsPanel

return OptionsPanel

