-- Defines the module locally
local TextAlerts = {}

-- Creates the text alerts tab
function TextAlerts.createTextAlertsTab(parent)
    local verticalSpacing = 30
    local basePosition = 10

    -- Creates checkboxes for enter and leave combat alerts
    local enterCB = _G.CombatAlert.Options.Controls.createCheckBox(
        parent,
        "Display text alert when entering combat",
        {x = basePosition, y = basePosition},
        "entercombatalert"
    )
    local leaveCB = _G.CombatAlert.Options.Controls.createCheckBox(
        parent,
        "Display text alert when leaving combat",
        {x = basePosition, y = basePosition + verticalSpacing},
        "leavecombatalert"
    )

    -- Creates text zoom slider
    local textZoomPosition = basePosition + (verticalSpacing * 2)
    local textZoomSlider, textZoomLabel, setZoomWithoutPreview =
        TextAlerts.createTextZoomSlider(parent, textZoomPosition)

    -- Creates color options for enter and leave combat
    local colorOptionsPosition = textZoomPosition + verticalSpacing + 16
    local colorOptions = _G.CombatAlert.Options.TextColors.dualcolorOptions(
        parent,
        480,
        "Enter Combat Color",
        "Leave Combat Color",
        _G.CombatAlert.Settings.get("enterColor") or
            _G.CombatAlert.Settings.defaults.enterColor,
        _G.CombatAlert.Settings.get("leaveColor") or
            _G.CombatAlert.Settings.defaults.leaveColor,
        function(newColor)
            _G.CombatAlert.Settings.set("enterColor", newColor)
        end,
        function(newColor)
            _G.CombatAlert.Settings.set("leaveColor", newColor)
        end
    )

    colorOptions:SetPosition(10, colorOptionsPosition)

    return {
        enterCB = enterCB,
        leaveCB = leaveCB,
        textZoomSlider = textZoomSlider,
        textZoomLabel = textZoomLabel,
        setZoomWithoutPreview = setZoomWithoutPreview,
        colorOptions = colorOptions
    }
end

-- Creates the text zoom slider
function TextAlerts.createTextZoomSlider(parent, position)
    local textLabel = _G.CombatAlert.Options.Controls.createLabel(
        parent,
        {80, 20},
        {10, position},
        "Text Zoom:"
    )
    local slider = Turbine.UI.Lotro.ScrollBar()
    slider:SetParent(parent)
    slider:SetSize(218, 10)
    slider:SetOrientation(Turbine.UI.Orientation.Horizontal)
    slider:SetPosition(95, position + 5)
    slider:SetMinimum(50)
    slider:SetMaximum(1800)
    slider:SetValue(_G.CombatAlert.Settings.get("textZoom") or 100)
    local valueLabel = _G.CombatAlert.Options.Controls.createLabel(
        parent,
        {50, 20},
        {320, position},
        slider:GetValue() .. "%"
    )

    -- Updates zoom value and UI
    local function updateZoom(zoomValue, triggerPreview)
        zoomValue = tonumber(zoomValue) or 100
        slider:SetValue(zoomValue)
        valueLabel:SetText(tostring(zoomValue) .. "%")
        _G.CombatAlert.Settings.set("textZoom", zoomValue)
        _G.CombatAlert.AlertUI.updateTextZoom(zoomValue)

        if triggerPreview then
            _G.CombatAlert.AlertLogic.previewAlert(false, true)
        end
    end

    local isManualAdjustment = true
    slider.ValueChanged = function(sender, args)
        if isManualAdjustment then
            updateZoom(slider:GetValue(), true)
        end
    end

    -- Sets zoom without triggering preview
    local function setZoomWithoutPreview(value)
        isManualAdjustment = false
        updateZoom(value, false)
        isManualAdjustment = true
    end

    setZoomWithoutPreview(
        tonumber(_G.CombatAlert.Settings.get("textZoom")) or 100
    )

    return slider, valueLabel, setZoomWithoutPreview
end

-- Updates the text zoom
function TextAlerts.updateTextZoom(zoom)
    _G.CombatAlert.AlertUI.updateTextZoom(zoom)
    _G.CombatAlert.Settings.set("textZoom", zoom)

    local centeredX, centeredY = TextAlerts.getCenteredAlertPosition()
    TextAlerts.updateAlertPosition(centeredX, centeredY)

    if not _G.CombatAlert.Options.OptionsPanel.isLoadingDefaults then
        _G.CombatAlert.AlertLogic.previewAlert(false, true)
    end
end

-- Updates the alert position
function TextAlerts.updateAlertPosition(x, y)
    _G.CombatAlert.AlertUI.updateAlertPosition(x, y)
    _G.CombatAlert.AlertLogic.previewAlert(false, true)
end

-- Gets the centered alert position
function TextAlerts.getCenteredAlertPosition()
    local windowWidth, windowHeight =
        _G.CombatAlert.AlertUI.getAlertWindowSize()
    local labelWidth, labelHeight =
        _G.CombatAlert.AlertUI.getAlertLabelSize()

    local centeredX = (windowWidth - labelWidth) / 2
    local centeredY = (windowHeight / 4) - (labelHeight / 2)

    return centeredX, centeredY
end

-- Makes TextAlerts globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Options = _G.CombatAlert.Options or {}
_G.CombatAlert.Options.TextAlerts = TextAlerts

return TextAlerts
