-- Defines the module locally
local TextColors = {}

-- Accesses the Controls module from the global CombatAlert namespace
local Controls = _G.CombatAlert.Options.Controls

-- Creates a color preview display
local function createColorPreview(parent, position, color)
    local colorDisplay = Turbine.UI.Control()
    colorDisplay:SetParent(parent)
    colorDisplay:SetSize(32, 32)
    colorDisplay:SetPosition(position.x, position.y)
    colorDisplay:SetBackColor(
        _G.CombatAlert.Utils.colorTupleToTurbineColor(color)
    )
    return colorDisplay
end

-- Creates a set of color sliders with a preview and callback
function TextColors.createColorSet(yOffset, text, currentValue, callback,
                                   previewFunction)
    local colorSet = Turbine.UI.Control()
    colorSet:SetSize(500, 128)
    colorSet:SetPosition(0, yOffset)

    local horizontalOffset = 18

    -- Creates color preview display
    local colorDisplay = createColorPreview(
        colorSet,
        {x = horizontalOffset, y = 0},
        currentValue
    )

    -- Creates label for the color set
    local label = Controls.createLabel(
        colorSet,
        {300, 32},
        {horizontalOffset + 42, 0},
        text
    )

    local sliders = {}
    local updateFunctions = {}
    local labels = {"A", "R", "G", "B"}

    -- Creates sliders for each color component
    for i = 1, 4 do
        local colorValue = math.floor((currentValue[i] or 0) * 255 + 0.5)
        local slider, updateFunc = Controls.createSlider(
            colorSet,
            labels[i],
            i,
            colorValue,
            horizontalOffset
        )
        sliders[i] = slider
        updateFunctions[i] = updateFunc
    end

    -- Updates color display and invokes callbacks
    local function updateColor()
        local newColor = {
            sliders[1]:GetValue() / 255,
            sliders[2]:GetValue() / 255,
            sliders[3]:GetValue() / 255,
            sliders[4]:GetValue() / 255
        }

        -- Ensures newColor is valid before using it
        if newColor and #newColor == 4 then
            colorDisplay:SetBackColor(
                _G.CombatAlert.Utils.colorTupleToTurbineColor(newColor)
            )
            callback(newColor)
            for _, updateFunc in ipairs(updateFunctions) do
                if type(updateFunc) == "function" then
                    updateFunc()
                end
            end
            -- Triggers the preview function if provided
            if previewFunction then
                previewFunction()
            end
        end
    end

    -- Attaches value changed event to each slider
    for _, slider in ipairs(sliders) do
        slider.ValueChanged = function()
            updateColor()
        end
    end

    -- Updates color function for external use
    colorSet.updateColor = function(self, newColor)
        if newColor and #newColor == 4 then
            for i = 1, 4 do
                local colorValue = math.floor((newColor[i] or 0) * 255 + 0.5)
                sliders[i]:SetValue(colorValue)
                if type(updateFunctions[i]) == "function" then
                    updateFunctions[i]()
                end
            end
            colorDisplay:SetBackColor(
                _G.CombatAlert.Utils.colorTupleToTurbineColor(newColor)
            )
            callback(newColor)
        end
    end

    -- Returns the sliders as part of the colorSet
    colorSet.aSlider = sliders[1]
    colorSet.rSlider = sliders[2]
    colorSet.gSlider = sliders[3]
    colorSet.bSlider = sliders[4]

    return colorSet
end

-- Creates dual color options
function TextColors.dualcolorOptions(parent, width, text1, text2,
                                     currentValue1, currentValue2, func1,
                                     func2)
    local optionsPanel = Turbine.UI.Control()
    optionsPanel:SetSize(width, 308)
    optionsPanel:SetMouseVisible(false)

    -- Creates two color sets
    local colorSet1 = TextColors.createColorSet(
        0, text1, currentValue1, func1,
        function() _G.CombatAlert.AlertLogic.previewAlert(true) end
    )
    local colorSet2 = TextColors.createColorSet(
        148, text2, currentValue2, func2,
        function() _G.CombatAlert.AlertLogic.previewAlert(false) end
    )

    if colorSet1 and colorSet2 then
        colorSet1:SetParent(optionsPanel)
        colorSet2:SetParent(optionsPanel)
    end

    -- Updates colors for both color sets
    optionsPanel.updateColors = function(self, newColor1, newColor2)
        if colorSet1 and colorSet1.updateColor then
            colorSet1:updateColor(newColor1)
        end
        if colorSet2 and colorSet2.updateColor then
            colorSet2:updateColor(newColor2)
        end
    end

    return optionsPanel
end

-- Makes TextColors globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Options = _G.CombatAlert.Options or {}
_G.CombatAlert.Options.TextColors = TextColors

return TextColors
