-- Imports custom CombatAlert modules
import "CombatAlert.Utils"
import "CombatAlert.Settings"

-- Defines the VisualAlerts module locally
local VisualAlerts = {}

-- Flags to manage initialization and programmatic changes
local isLoadingDefaults = false
local isProgrammaticChange = false
local isInitializing = true

-- Retrieves the default color from settings
local function getDefaultColor()
    return _G.CombatAlert.Settings.defaults.alertColorPresets.Default
end

-- Creates the Visual Alerts tab
function VisualAlerts.createVisualAlertsTab(parent)
    local visualBasePosition = 10
    local visualVerticalSpacing = 30

    -- Creates visual alerts options and controls
    local colorDropDown, dropDownArrow, colorDropDownLabel,
          selectedValueLabel, colorListBox =
          VisualAlerts.createVisualAlertsOptions(parent,
              {x = visualBasePosition, y = visualBasePosition})

    -- Creates checkbox for enabling visual alerts
    local visualCB = _G.CombatAlert.Options.Controls.createCheckBox(
        parent, "Enable visual alerts",
        {x = visualBasePosition, y = visualBasePosition +
        visualVerticalSpacing}, "visualalert")

    -- Adjusted position for custom visual alert color control
    local customColorPosition = visualBasePosition +
        (visualVerticalSpacing * 2) + 16

    local customColorOptions = _G.CombatAlert.Options.TextColors
        .createColorSet(customColorPosition, "Custom Visual Alert Color",
        _G.CombatAlert.Settings.get("alertColorPresets").Custom,
        function(newColor)
            if not isLoadingDefaults and not isProgrammaticChange and
               not isInitializing then
                _G.CombatAlert.Settings.get("alertColorPresets").Custom =
                    newColor

                -- Switches dropdown to "Custom" only if manually adjusting
                if selectedValueLabel:GetText() ~= "Custom" then
                    selectedValueLabel:SetText("Custom")
                end

                _G.CombatAlert.Settings.set("alertColor", "Custom")

                -- Updates alert background
                _G.CombatAlert.AlertUI.updateAlertBackground()

                -- Triggers enter combat preview
                _G.CombatAlert.AlertLogic.previewAlert(true)
            end
        end)

    if customColorOptions then
        customColorOptions:SetParent(parent)
        customColorOptions:SetPosition(visualBasePosition, customColorPosition)
        _G.CombatAlert.Options.OptionsPanel =
            _G.CombatAlert.Options.OptionsPanel or {}
        _G.CombatAlert.Options.OptionsPanel.customColorOptions =
            customColorOptions

        -- Modifies the "A" slider to act as the opacity slider
        customColorOptions.aSlider.ValueChanged = function(sender, args)
            local opacity = sender:GetValue()
            _G.CombatAlert.Settings.set("opacity", opacity)
            _G.CombatAlert.AlertUI.updateAlertBackground()

            -- Previews alert if not loading defaults
            if not isLoadingDefaults then
                _G.CombatAlert.AlertLogic.previewAlert(true)
            end
        end
    end

    -- Updates UI based on saved settings
    local function updateUIFromSettings()
        local savedAlertColor = _G.CombatAlert.Settings.get("alertColor")
                                or "Default"
        selectedValueLabel:SetText(savedAlertColor)

        local colorValues
        if savedAlertColor == "Custom" then
            colorValues = _G.CombatAlert.Settings.get("alertColorPresets")
                          .Custom
        else
            colorValues = VisualAlerts.getPresetColor(savedAlertColor)
        end

        isProgrammaticChange = true
        VisualAlerts.applyPresetColor(colorValues)
        isProgrammaticChange = false
    end

    -- Calls updateUIFromSettings initially
    updateUIFromSettings()

    -- Stores updateUIFromSettings function for external access
    VisualAlerts.updateUIFromSettings = updateUIFromSettings

    -- Sets initialization flag to false after setup
    isInitializing = false

    return { colorDropDown = colorDropDown, dropDownArrow = dropDownArrow,
             selectedValueLabel = selectedValueLabel, colorListBox = colorListBox,
             visualCB = visualCB, customColorOptions = customColorOptions,
             updateUIFromSettings = updateUIFromSettings }
end

-- Creates a dropdown for visual alert graphic selection
function VisualAlerts.createVisualAlertsOptions(parent, position)
    local verticalSpacing = 30
    local baseY = position.y

    -- Creates dropdown for selecting visual alert graphic
    local dropdown, dropDownArrow, colorLabel, selectedValueLabel =
        _G.CombatAlert.Options.Controls.createDropdown(parent,
        "Visual Alert Color:", {x = position.x, y = baseY}, 190)

    -- Creates color selection list box
    local listBox = VisualAlerts.createColorListBox(parent, dropdown,
                                                    selectedValueLabel)

    -- Toggles list box visibility on dropdown click
    dropdown.MouseClick = function(sender, args)
        if listBox then
            listBox:SetVisible(not listBox:IsVisible())
            if listBox:IsVisible() then
                listBox:SetZOrder(1000)
            end
        end
    end

    dropDownArrow.MouseClick = dropdown.MouseClick

    return dropdown, dropDownArrow, colorLabel, selectedValueLabel, listBox
end

-- Creates a listbox for color selection
function VisualAlerts.createColorListBox(parent, dropdown, selectedValueLabel)
    local listBox = Turbine.UI.ListBox()
    listBox:SetParent(parent)
    listBox:SetSize(190, 220)  -- Adjusted height for one less color
    listBox:SetPosition(dropdown:GetLeft(), dropdown:GetTop() +
                         dropdown:GetHeight())
    listBox:SetBackColor(Turbine.UI.Color(1, 0.1, 0.1, 0.1))
    listBox:SetVisible(false)
    listBox:SetZOrder(1000)

    -- Retrieves all color presets
    local presets = _G.CombatAlert.Settings.get("alertColorPresets")

    -- Defines available colors for selection
    local colors = {}
    for name, value in pairs(presets) do
        table.insert(colors, {name, value})
    end

    -- Sorts colors with "Default" first and "Custom" last
    table.sort(colors, function(a, b)
        if a[1] == "Default" then return true
        elseif b[1] == "Default" then return false
        elseif a[1] == "Custom" then return false
        elseif b[1] == "Custom" then return true
        else return a[1] < b[1] end
    end)

    -- Creates list items for each color
    for _, colorData in ipairs(colors) do
        local color, value = colorData[1], colorData[2]
        local item = Turbine.UI.Label()
        item:SetSize(170, 20)
        item:SetFont(Turbine.UI.Lotro.Font.TrajanPro14)
        item:SetForeColor(Turbine.UI.Color(0.8, 0.75, 0.5))
        item:SetText(color)
        item:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        item:SetMouseVisible(true)

        -- Sets selected color and updates settings on item click
        item.MouseClick = function(sender, args)
            selectedValueLabel:SetText(color)
            listBox:SetVisible(false)
            _G.CombatAlert.Settings.set("alertColor", color)

            if color ~= "Custom" then
                isProgrammaticChange = true
                VisualAlerts.applyPresetColor(value)
                isProgrammaticChange = false
            else
                -- For Custom, use the stored custom color
                local customColor = _G.CombatAlert.Settings.get("alertColorPresets")
                    .Custom
                VisualAlerts.applyPresetColor(customColor)
            end

            -- Updates alert background
            _G.CombatAlert.AlertUI.updateAlertBackground()

            -- Triggers enter combat preview
            _G.CombatAlert.AlertLogic.previewAlert(true)
        end

        listBox:AddItem(item)
    end

    -- Sets default selected color
    selectedValueLabel:SetText(_G.CombatAlert.Settings.get("alertColor")
                               or "Default")

    return listBox
end

-- Applies preset color values to the sliders
function VisualAlerts.applyPresetColor(colorValues)
    local optionsPanel = _G.CombatAlert.Options.OptionsPanel
    local customColorOptions = optionsPanel and optionsPanel.customColorOptions

    if customColorOptions then
        -- Updates sliders to match preset values
        customColorOptions.aSlider:SetValue(math.floor(colorValues[1] * 255))
        customColorOptions.rSlider:SetValue(math.floor(colorValues[2] * 255))
        customColorOptions.gSlider:SetValue(math.floor(colorValues[3] * 255))
        customColorOptions.bSlider:SetValue(math.floor(colorValues[4] * 255))

        -- Updates the color preview
        customColorOptions.updateColor(colorValues)
    end
end

-- Gets preset color values
function VisualAlerts.getPresetColor(colorName)
    local presets = _G.CombatAlert.Settings.get("alertColorPresets")
    return presets[colorName] or presets.Default
end

-- Toggles the dropdown (for use in OptionsPanel)
function VisualAlerts.toggleDropdown(optionsPanel)
    if optionsPanel.colorListBox then
        local isVisible = optionsPanel.colorListBox:IsVisible()
        optionsPanel.colorListBox:SetVisible(not isVisible)
        if optionsPanel.colorListBox:IsVisible() then
            optionsPanel.colorListBox:SetZOrder(1000)
        end
    end
end

-- Updates sliders and dropdown when loading defaults
function VisualAlerts.loadDefaults()
    isLoadingDefaults = true
    local optionsPanel = _G.CombatAlert.Options.OptionsPanel

    if optionsPanel and optionsPanel.selectedValueLabel then
        optionsPanel.selectedValueLabel:SetText("Default")
    end

    -- Applies the default color with the specified ARGB values
    VisualAlerts.applyPresetColor(getDefaultColor())

    -- Sets the custom color to match the default
    _G.CombatAlert.Settings.get("alertColorPresets").Custom =
        getDefaultColor()

    isLoadingDefaults = false
end

-- Makes VisualAlerts globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Options = _G.CombatAlert.Options or {}
_G.CombatAlert.Options.VisualAlerts = VisualAlerts

return VisualAlerts
