-- Define the module locally
local Utils = {}

-- Adds a callback function to an object's event
function Utils.addCallback(object, event, callback)
    if object[event] == nil then
        object[event] = callback
    else
        if type(object[event]) == "table" then
            table.insert(object[event], callback)
        else
            object[event] = {object[event], callback}
        end
    end
    return callback
end

-- Removes a callback function from an object's event
function Utils.removeCallback(object, event, callback)
    if object[event] == callback then
        object[event] = nil
    else
        if type(object[event]) == "table" then
            local size = table.getn(object[event])
            for i = 1, size do
                if object[event][i] == callback then
                    table.remove(object[event], i)
                    break
                end
            end
        end
    end
end

-- Converts a color tuple to a Turbine.UI.Color object
function Utils.colorTupleToTurbineColor(tuple)
    -- Returns transparent color if no tuple
    if not tuple then
        return Turbine.UI.Color(0, 0, 0, 0)
    end

    local function createColor(t)
        -- Returns transparent color if invalid
        if type(t) == "table" and #t == 4 then
            return Turbine.UI.Color(t[1], t[2], t[3], t[4])
        end
        return Turbine.UI.Color(0, 0, 0, 0)
    end

    if type(tuple[1]) == "table" then
        local colorTable = {}
        for key, value in pairs(tuple) do
            colorTable[key] = createColor(value)
        end
        return colorTable
    else
        return createColor(tuple)
    end
end

-- Creates a deep copy of a table or value
function Utils.deepCopy(orig)
    if type(orig) ~= "table" then return orig end
    local copy = {}
    for origKey, origValue in pairs(orig) do
        copy[Utils.deepCopy(origKey)] = Utils.deepCopy(origValue)
    end
    return setmetatable(copy, getmetatable(orig))
end

-- Attempts to load a background image
function Utils.tryLoadBackground(parent, path)
    local testImage = Turbine.UI.Control()
    testImage:SetParent(parent)
    testImage:SetPosition(0, 0)
    testImage:SetSize(1100, 500)
    testImage:SetMouseVisible(false)

    local success = pcall(function()
        testImage:SetBackground(path)
    end)

    -- Sets stretch mode for the image
    if success and testImage:GetBackground() then
        testImage:SetStretchMode(1)
        testImage:SetSize(parent:GetWidth(), parent:GetHeight())
        return true, testImage
    else
        -- Returns failure if image loading fails
        return false, nil
    end
end

-- Applies color to a label for enter and leave combat alerts
function Utils.applyColor(label, colorTable)
    local alpha = colorTable[1] or 1
    local red = colorTable[2] or 1
    local green = colorTable[3] or 1
    local blue = colorTable[4] or 1

    local textColor = Turbine.UI.Color(1, red, green, blue)
    label:SetForeColor(textColor)
    label:SetOpacity(alpha)

    return textColor
end

-- Makes Utils globally accessible
_G.CombatAlert = _G.CombatAlert or {}
_G.CombatAlert.Utils = Utils

return Utils